// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.collections;

import java.util.Iterator;
import java.util.SortedSet;
import xl2.collections.node.SkipNode;

  /**
  * The entry set of a SkipMap.
  * @author Aaron Hansen
  * @see SkipMap#entrySet()
  * @version 1.0.0
  */
class EntrySet extends SubSkipSet {

  // Constructors
  // ------------

  public EntrySet() {}

  public EntrySet(SkipSet set) {
    super(set);
    }

  public EntrySet(SkipSet set, Object from, Object to) {
    super(set,from,to);
    }


  // Public Methods
  // --------------

  public boolean add(Object obj) {
    throw new UnsupportedOperationException();
    }

  public boolean contains(Object obj) {
    if (obj instanceof SkipNode)
      return super.contains(((SkipNode)obj).getKey());
    else
      return super.contains(obj);
    }

  public Object get(Object key) {
    if (inRange(key)) {
      SkipNode node = getSet().nodeOf(key);
      if (node != null)
        return node.getValue();
      }
    return null;
    }

  public Iterator iterator() {
    return new SkipNodeIterator(getSet(),floor(),ceiling());
    }

  public Object put(Object key, Object value) {
    if (inRange(key))
      return getSet().put(key,value);
    return null;
    }
 
  public boolean remove(Object obj) {
    if (obj instanceof SkipNode)
      return super.remove(((SkipNode)obj).getKey());
    else
      return super.remove(obj);
    }

  public Object removeKey(Object key) {
    if (inRange(key))
      return getSet().removeKey(key);
    return null;
    }

  public SortedSet subSet(Object from, Object to) {  
    if (!inRange(from))
      throw new IllegalArgumentException("From out of range.");
    if (!inRange(to))
      throw new IllegalArgumentException("To out of range.");
    if ((from != null) && (to != null) && (getSet().compare(from,to) > 0))
      throw new IllegalArgumentException("From is greater than To.");
    return new EntrySet(getSet(),from,to);
    }


  }//EntrySet
