// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.collections;

import java.io.*;
import java.util.*;
import xl2.collections.node.ListNode;

  /**
  * @author Aaron Hansen
  * @see SList
  * @version 1.0.0
  */
class SListIterator implements ListIterator {

  // Protected Fields
  // ----------------

  protected boolean FailFastBoolean = true;

  protected ListNode FirstExcludedNode = null;

  protected ListNode LastNode = null;

  protected transient int ModInt = 0;

  protected SList MyList = null;

  protected int NextIndexInt = 0;

  protected ListNode NextNode = null;

  protected boolean RemovedBoolean = false;

  
  // Constructors
  // ------------

  public SListIterator() {}

  public SListIterator(SList list, int next, int exclude) {
    MyList = list;
    ModInt = list.ModInt;
    if (next <= 0) {
      NextIndexInt = 0;
      NextNode = list.HeadNode;
      }
    else {
      NextIndexInt = next;
      NextNode = list.getNode(next);
      }
    if ((exclude >= 0) && (exclude < list.size()))
      FirstExcludedNode = list.getNode(exclude);
    }


  // Public Methods
  // --------------

    /**
    * Can not call after remove.
    */
  public void add(Object obj) {
    if (RemovedBoolean)
      throw new IllegalStateException("Attempt to add after removal.");
    ListNode newNode = MyList.addAfter(obj,LastNode);
    if (newNode != null) {
      LastNode = newNode;
      NextIndexInt++;
      ModInt++;
      }
    }

  public boolean getFailFast() {
    return FailFastBoolean;
    }

  public boolean hasNext() {
    return NextNode != FirstExcludedNode;
    }

    /**
    * @throws UnsupportedOperationException
    */
  public boolean hasPrevious() {
    throw new UnsupportedOperationException();
    }

  public Object next() {
    if (NextNode == FirstExcludedNode)
      throw new NoSuchElementException();
    if ((MyList.ModInt != ModInt) && (FailFastBoolean))
      throw new ConcurrentModificationException();
    LastNode = NextNode;
    NextNode = NextNode.NextNode;
    NextIndexInt++;
    RemovedBoolean = false;
    return LastNode.DataObject;
    }
    
  public int nextIndex() {
    return NextIndexInt;
    }

  public void remove() {
    if (LastNode == null)
      throw new IllegalStateException("Nothing to remove.");
    if ((MyList.ModInt != ModInt) && (FailFastBoolean))
      throw new ConcurrentModificationException();
    if (MyList.removeNode(LastNode)) {
      NextIndexInt--;
      ModInt++;
      LastNode = null;
      RemovedBoolean = true;
      }
    }

  public Object previous() {
    throw new UnsupportedOperationException();
    }

  public int previousIndex() {
    throw new UnsupportedOperationException();
    }

  public void set(Object object) {
    if (LastNode == null)
      throw new IllegalStateException("Nothing to replace.");
    LastNode.DataObject = object;
    }

  public void setFailFast(boolean b) {
    FailFastBoolean = b;
    }


  }//SListIterator
