// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.collections;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;
import xl2.collections.node.SkipNode;

  /**
  * SkipSet iterator which returns SkipNodes.
  * @author Aaron Hansen
  * @see SkipSet
  * @version 1.0.0
  */
class SkipNodeIterator implements Externalizable, Iterator {

  // Protected Fields
  // ----------------

  protected boolean FailFastBoolean = true;

  protected SkipNode FirstExcludedNode = null;

  protected SkipNode LastNode = null;

  protected int ModInt = 0;

  protected SkipSet MySet = null;

  protected SkipNode NextNode = null;


  // Constructors
  // ------------

  public SkipNodeIterator() {}

  public SkipNodeIterator(SkipSet set, SkipNode floor, SkipNode ceil) {
    MySet = set;
    ModInt = set.ModInt;
    NextNode = floor;
    FirstExcludedNode = ceil;
    }


  // Public Methods
  // --------------

  public boolean getFailFast() {
    return FailFastBoolean;
    }

  public boolean hasNext() {
    return NextNode != FirstExcludedNode;
    }

  public Object next() {
    if (NextNode == FirstExcludedNode)
      throw new NoSuchElementException();
    if ( (MySet.ModInt != ModInt) && (FailFastBoolean) )
      throw new ConcurrentModificationException();
    LastNode = NextNode;
    NextNode = NextNode.Next[0];
    return LastNode;
    }
    
  public void readExternal(ObjectInput in) 
    throws ClassNotFoundException, IOException {
    MySet = (SkipSet) in.readObject();
    FirstExcludedNode = (SkipNode) in.readObject();
    LastNode = (SkipNode) in.readObject();
    NextNode = (SkipNode) in.readObject();
    FailFastBoolean = in.readBoolean();
    ModInt = in.readInt();
    }

  public void remove() {
    if (LastNode == null)
      throw new IllegalStateException();
    if ( (MySet.ModInt != ModInt) && (FailFastBoolean) )
      throw new ConcurrentModificationException();
    MySet.remove(LastNode.getKey());
    ModInt++;
    LastNode = null;
    }

  public void setFailFast(boolean b) {
    FailFastBoolean = b;
    }

  public void writeExternal(ObjectOutput out) throws IOException {
    out.writeObject(MySet);
    out.writeObject(FirstExcludedNode);
    out.writeObject(LastNode);
    out.writeObject(NextNode);
    out.writeBoolean(FailFastBoolean);
    out.writeInt(ModInt);
    }


  }//SkipNodeIterator
