// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.collections.node;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.*;

  /**
  * Node for SkipSet.  
  * Implements Map.Entry so that it can easily be subclassed for SkipMap.
  * @author Aaron Hansen
  * @see SkipSet
  * @version 1.0.0
  */
public class SkipNode implements Externalizable, Map.Entry {

  // Public Fields
  // ----------------

  public SkipNode[] Next = null;

  public Object Key = null;


  // Constructors
  // ------------

  public SkipNode(int levels, Object key) {
    Next = new SkipNode[levels];
    Key = key;
    }


  // Public Methods
  // --------------

  public boolean equals(Object obj) {
    if (!(obj instanceof Map.Entry))
      return false;
    Map.Entry e = (Map.Entry)obj;
    return (getKey()==null ? e.getKey()==null : Key.equals(e.getKey())) &&
      (getValue()==null ? e.getValue()==null : getValue().equals(e.getValue()));
    }

  public Object getKey() {
    return Key;
    }

  public Object getValue() {
    return Key;
    }

  public int hashCode() {
    int keyHash = (getKey()==null ? 0 : getKey().hashCode());
    int valueHash = (getValue()==null ? 0 : getValue().hashCode());
    return keyHash ^ valueHash;
    }

  public void readExternal(ObjectInput in) 
    throws ClassNotFoundException, IOException {
    Next = (SkipNode[]) in.readObject();
    Key = in.readObject();
    }

  public Object setValue(Object obj) {
    throw new UnsupportedOperationException();
    }

  public void writeExternal(ObjectOutput out) throws IOException {
    out.writeObject(Next);
    out.writeObject(Key);
    }


  }//SkipNode
