import xl2.odb.*;

  /**
  * The object being stored in the database.
  */
public class Person implements java.io.Serializable {

  // Protected Fields
  // ----------------

  protected XL2Reference[] Children = null;

  protected XL2Reference Father = null;

  protected XL2Reference Mother = null;

  protected String Name = null;


  // Constructors
  // ------------

  public Person() {}

  public Person(String name) {
    setName(name);
    }


  // Public Methods
  // --------------

    /**
    * Adds non-null children.
    */
  public void addChild(Person child) {
    if (child == null)
      return;
    int num = 0;
    if (Children == null) {
      Children = new XL2Reference[1]; 
      Children[0] = new XL2Reference(this); //automatically locks this
      Children[0].set(child); //automatically locks this too
      }
    else {
      num = Children.length;
      XL2Reference[] temp = new XL2Reference[num + 1];
      System.arraycopy(Children,0,temp,0,num);
      temp[num] = new XL2Reference(this);
      temp[num].set(child);
      Children = temp;
      }
    }

    /**
    * Finds a person by name in the tree rooted from me.  Traverses the tree 
    * in preorder and returns the first encountered.
    */
  public Person find(String name) {
    if (Name.equals(name))
      return this;
    if (Children == null)
      return null;
    Person found = null;
    int len = Children.length;
    for (int i = 0; i < len; i++) {
      found = ((Person)Children[i].get()).find(name);
      if (found != null)
        return found;
      }
    return null;
    }

    /**
    * Will not return null, but possibly an empty array.
    */
  public Person[] getChildren() {
    int len = 0;
    if (Children != null)
      len = Children.length;
    Person[] people = new Person[len];
    for (int i = 0; i < len; i++)
      people[i] = (Person) Children[i].get();
    return people;
    }

  public Person getFather() {
    if (Father == null)
      return null;
    return (Person) Father.get();
    }

  public Person getMother() {
    if (Mother == null)
      return null;
    return (Person) Mother.get();
    }

  public String getName() {
    return Name;
    }

    /**
    * Prints the tree (names) to System.out in preorder.
    */
  public void print() {
    print("");
    }

  public void setMother(Person parent) {
    if (Mother == null)
      Mother = new XL2Reference(this);
    Mother.set(parent);
    }

  public void setFather(Person parent) {
    if (Father == null)
      Father = new XL2Reference(this);
    Father.set(parent);
    }

    /**
    * Modifies a second class object, so this method explicitly attempts 
    * to lock the person.
    */
  public void setName(String name) {
    XL2TransactionMgr.writeLock(this); //always attempt lock b4 modification
    Name = name;
    }

  public String toString() {
    return Name;
    }


  // Private Methods
  // ---------------

  public void print(String indent) {
    System.out.print(indent);
    System.out.println(getName());
    if (Children == null)
      return;
    XL2Reference child;
    indent = indent + "  ";
    int len = Children.length;
    for (int i = 0; i < len; i++) {
      child = Children[i];
      if (child != null)
        ((Person)child.get()).print(indent);
      }
    }


  }//Person
