// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import xl2.transaction.TransactionAbortedException;

  /**
  * Compacts unused space in a database.
  * @author Aaron Hansen
  * @version 1.1.0
  */
class Compactor extends XL2Transaction {

  // Constructors
  // ------------

  Compactor(XL2Database database) {
    super(database);
    }


  // Public Methods
  // --------------

  public boolean hasWork() {
    return !getDatabase().ObjFreeList.isEmpty();
    }

  // Protected Methods
  // -----------------

  protected void doCommit() {
    XL2Database d = getDatabase();
    d.CloseQueue.enqueue();
    try {
      FreeEntry entry = d.ObjFreeList.first();
      if (entry == null)
        return;
      long handle = 0;
      synchronized (d.LockObject) {
        d.ObjectReader.seek(entry.next());
        handle = d.ObjectReader.readLong();
        }
      Object obj = d.get(handle);
      d.ObjFreeList.remove(entry);
      Referent ref = ReferentMgr.get(obj);
      byte[] bytes = new byte[ref.getLength()];
      synchronized (d.LockObject) {
        d.ObjectReader.seek(ref.getOffset() + 8);
        d.ObjectReader.read(bytes);
        d.ObjectWriter.seek(entry.OffsetLong);
        d.ObjectWriter.writeLong(ref.getHandle());
        d.ObjectWriter.write(bytes);
        d.ObjectWriter.flush();
        }
      ref.setOffset(entry.OffsetLong);
      entry.OffsetLong += (ref.getLength() + 8);
      d.ObjFreeList.add(entry);
      d.saveState(ref);
      super.doCommit();
      }
    catch (TransactionAbortedException x) {
      return;
      }
    catch (Exception x) {
      System.err.println("Report this stack trace to feedback@xl2.net");
      x.printStackTrace();
      abort();
      }
    }


  }//Compactor

/*
History
-------

v1.1.0, 7-24-2000
  - Synchronized file reading and writing.
*/
