// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

  /**
  * Encypts serialized objects using Password Based Encryption with MD5 and
  * DES.  Uses the Java Cryptography Extension.
  * <p>
  * Be sure to set database password #setPassword(String) before opening a
  * database.
  * <p>
  * The <a href="http://java.sun.com/products/jce/">JCE</a> must be 
  * installed in the runtime environment. 
  * @author Aaron Hansen
  * @version 1.1.0
  */
public class EncryptedDatabase extends XL2Database {

  static {
    Security.addProvider(new com.sun.crypto.provider.SunJCE());
    }

  // Protected Fields
  // ----------------

  protected Cipher TheCipher = null;

  protected SecretKey TheKey = null;

  protected PBEParameterSpec TheParam = null;


  // Constructors
  // ------------

  public EncryptedDatabase() {
    try {
      byte[] salt = { 
        (byte)0xc7, (byte)0x73, (byte)0x21, (byte)0x8c,
        (byte)0x7e, (byte)0xc8, (byte)0xee, (byte)0x99
        };
      TheParam = new PBEParameterSpec(salt, 20);
      }
    catch (Exception x) {
      throw new XL2Exception(x);
      }
    }


  // Public Methods
  // --------------

    /**
    * Returns an ObjectInputStream nesting a CipherInputStream.
    * @throws XL2Exception Wrapping any exceptions from the crypto lib.
    */
  public ObjectInput getObjectInput(InputStream in) throws XL2Exception {
    try {
      TheCipher.init(Cipher.DECRYPT_MODE,TheKey,TheParam);
      return super.getObjectInput(new CipherInputStream(in,TheCipher));
      }
    catch (Exception x) {
      throw new XL2Exception(x);
      }
    }

    /**
    * Returns an ObjectOutputStream nesting a CipherOutputStream.
    * @throws XL2Exception Wrapping any exceptions from the crypto lib.
    */
  public ObjectOutput getObjectOutput(OutputStream out) throws XL2Exception {
    try {
      TheCipher.init(Cipher.ENCRYPT_MODE,TheKey,TheParam);
      return super.getObjectOutput(new CipherOutputStream(out,TheCipher));
      }
    catch (Exception x) {
      throw new XL2Exception(x);
      }
    }

    /**
    * Will throw an exception, use newDatabase(String,String)
    */
  public void newDatabase(String dbDir) throws XL2Exception {
    throw new XL2Exception("Unsupported Operation");
    }

    /**
    * Creates an EncryptedDatabase that will be closed upon 
    * completion of this method.
    * @param dbDir XL2Database directory.
    * @param password Key for encrypting/decrypting data.
    * @throws XL2Exception Wrapping any internal exception such as an 
    *                      IOException.
    */
  public void newDatabase(String dbDir, String password) throws XL2Exception {
    try {
      setPassword(password);
      super.newDatabase(dbDir);
      }
    catch (XL2Exception x) {
      throw x;
      }
    catch (Exception x) {
      throw new XL2Exception(x);
      }
    }

    /**
    * Password must have be set before calling this:
    * Opens the database with the given mode.
    * @param dbDir The database directory.
    * @param mode OPEN_READ_ONLY or OPEN_READ_WRITE
    * @throws DatabaseLockedException If a lock file prevents opening.
    * @throws XL2Exception Wrapping any exceptions.
    */
  public void open(String dbDir, int mode) 
    throws DatabaseLockedException, XL2Exception {
    if (TheKey == null)
      throw new XL2Exception(new IllegalStateException(
        "Attempt to open an encrypted database without a password."));
    super.open(dbDir,mode);
    }

    /**
    * Must be called before opening a database.
    * @throws XL2Exception Wrapping any exceptions.
    */
  public void setPassword(String password) throws XL2Exception {
    if (getOpenMode() != NOT_OPEN)
      throw new XL2Exception(new IllegalStateException(
        "Attempt to set password of an open database."));
    try {
      TheKey = SecretKeyFactory.
        getInstance("PBEWithMD5AndDES").
          generateSecret(new PBEKeySpec(password.toCharArray()));
      TheCipher = Cipher.getInstance("PBEWithMD5AndDES");
      }
    catch (Exception x) {
      throw new XL2Exception(x);
      }
    }


  }//EncryptedDatabase

/*
History
-------

Version 1.1.0 (7/22/00)
  Changed database creation.
    - Added public void create
    - Removed public static synchronized XL2Database create


*/
