// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.io.*;
import java.util.*;
import xl2.collections.SkipSet;

  /**
  * For the garbage and free handle stacks.
  * @author Aaron Hansen
  * @version 1.1.0
  */
class FreeStack implements Comparator {

  // Protected Fields
  // ----------------

  protected boolean ModifiedBoolean = false;

  protected long OffsetLong = 0;

  protected SkipSet PushSet = null;

  protected RandomAccessFile TheFile = null;


  // Constructors
  // ------------

  public FreeStack(File file) throws IOException {
    TheFile = new RandomAccessFile(file,"rw");
    init();
    }


  // Public Methods
  // --------------

  public void abort() throws IOException {
    init();
    }

  public void close() throws IOException {
    if (TheFile != null)
      TheFile.close();
    }

  public void commit() throws IOException {
    if (!ModifiedBoolean)
      return;
    OffsetLong += 8;
    TheFile.setLength(OffsetLong);
    TheFile.seek(OffsetLong);
    Iterator iterator = PushSet.iterator();
    Long l;
    while (iterator.hasNext()) {
      l = (Long) iterator.next();
      TheFile.writeLong(l.longValue());
      }
    init();
    }

  public int compare(Object o1, Object o2) {
    long l = ((Long)o2).longValue() - ((Long)o1).longValue(); //yep, reverse
    if (l > 0)
      return 1;
    if (l < 0)
      return -1;
    return 0;
    }

    /**
    * The file descriptor of the file being used.
    */
  public FileDescriptor getFD() throws IOException {
    return TheFile.getFD();
    }

  public boolean isEmpty() {
    return (OffsetLong < 0);
    }

  public long pop() throws IOException {
    if (isEmpty())
      throw new IllegalStateException("Empty stack.");
    ModifiedBoolean = true;
    long l  = TheFile.readLong();
    OffsetLong -= 8L;
    if (OffsetLong >= 0)
      TheFile.seek(OffsetLong);
    return l;
    }

  public void push(long l) {
    ModifiedBoolean = true;
    PushSet.add(new Long(l));
    }


  // Protected Methods
  // -----------------

  protected void init() throws IOException {
    OffsetLong = TheFile.length() - 8L;
    if (OffsetLong >= 0)
      TheFile.seek(OffsetLong);
    if (PushSet == null)
      PushSet = new xl2.odb.util.HashSet();
    else
      PushSet.clear();
    ModifiedBoolean = false;
    }


  }//FreeStack

/*
History
-------

v1.1.0, 7-24-2000
  - Added getFD for syncing purposes.

*/
