// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

  /**
  * @author Aaron Hansen
  * @version 1.0.0
  */
class GarbageCollector extends FreeStack {

  // Protected Fields
  // ----------------

  protected XL2Database TheDatabase = null;


  // Constructors
  // ------------

  public GarbageCollector(XL2Database db, File file) throws IOException {
    super(file);
    TheDatabase = db;
    }


  // Public Methods
  // --------------

  public void add(Referent referent) {
    this.push(referent.getHandle());
    }

  public void gc() {
    XL2Database d = TheDatabase;
    d.CloseQueue.enqueue();
    XL2Transaction t = XL2TransactionMgr.currentTransaction();
    if (!t.isOpen())
      return;
    try {
      long l = pop();
      Object o = d.get(l);
      Referent ref = (Referent) d.TheHandleSet.get(l);
      if (ref == null) {
        return;
        }
      else if (ref.getRefCount() > 0) {
        d.log("Invalid object in the garbage stack!",null);
        return;
        }
      t.saveState(o);
      d.HandleFreeStack.push(ref.getHandle());
      d.ObjFreeList.add(ref.getOffset(),(ref.getLength() + 8));
      ref.setStored(false);
      Set set = referents(o);
      Iterator iterator = set.iterator();
      while (iterator.hasNext()) {
        ref = ReferentMgr.get(iterator.next());
        ref.decRefCount();
        }
      }
    catch (IllegalAccessException iae) {
      d.log(null,iae);
      System.err.println(
        "XL2 requires security permissions to reflect private fields.");
      t.abort();
      }
    catch (Exception x) {
      d.log(null,x);
      t.abort();
      }
    }

    /**
    * Find all second class XL2References of the first class object.
    */
  public static Set referents(Object obj) throws IllegalAccessException {
    return referents(obj, 
                     new xl2.odb.util.HashSet(), 
                     new xl2.odb.util.HashSet());
    }

    /**
    * Find all second class XL2References of the first class object.
    */
  public static Set referents(Object object, Set found, Set checked) 
    throws IllegalAccessException {
    if (object instanceof XL2Reference) {
      found.add(((XL2Reference)object).get());
      return found;
      }
    else if (object instanceof Object[]) {
      Object[] objects = (Object[]) object;
      for (int i = objects.length; --i >= 0; ) {
        if ((objects[i] != null) && !checked.contains(objects[i])) {
          checked.add(objects[i]);
          referents(objects[i],found,checked);
          }
        }
      return found;
      }
    Object field;
    Field[] fields;
    int i = 0;
    Class objClass = object.getClass();
    while (objClass != null) {
      fields = objClass.getDeclaredFields();
      i = fields.length;
      if (i > 0)
        AccessibleObject.setAccessible(fields,true);
      while (--i >= 0) {
        field = fields[i].get(object);
        if (field instanceof XL2Reference)
          found.add(((XL2Reference)field).get());
        else if ((field != null) 
          && !checked.contains(field)
          && !fields[i].getType().isPrimitive()) {
          checked.add(field);
          referents(field,found,checked);
          }
        }
      objClass = objClass.getSuperclass();
      }
    return found;
    }


  }//Garbage
