// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.lang.ref.ReferenceQueue;
import xl2.odb.util.ReferenceSet;
import xl2.odb.util.WeakHashReference;

  /**
  * @author Aaron Hansen
  * @version 1.0.0
  */
class LockMgr implements XL2Constants {

  // Protected Fields
  // ----------------

  protected static HandleSet TheHandleSet = new HandleSet();

  protected static ReferenceSet TheReferenceSet = new ReferenceSet();

  protected static ReferenceQueue TheReferenceQueue = new ReferenceQueue();


  // Constructors
  // ------------

  private LockMgr() {}


  // Public Methods
  // --------------

  public static long getLockMode(Object obj, long handle) {
    HandleEntry entry = (HandleEntry) TheHandleSet.get(handle);
    if (entry == null)
      entry = (HandleEntry) TheReferenceSet.referenceOf(obj);
    if (entry == null)
      return -1;
    return entry.ModeInt;
    }

    /**
    * Not to be called more than once by the locking transaction for a 
    * specific object.
    */
  public static synchronized void lock(Object obj, long handle, int mode) {
    HandleEntry entry = (HandleEntry) TheHandleSet.get(handle);
    if (entry == null)
      entry = (HandleEntry) TheReferenceSet.referenceOf(obj);
    if (entry == null) {
      entry = new HandleEntry(obj,handle,mode);
      if (handle < 0)
        TheReferenceSet.add(entry);
      else
        TheHandleSet.add(entry);
      return;
      }
    if ((mode == WRITE) || (entry.ModeInt == WRITE))
      throw new LockNotGrantedException(
        obj,LockNotGrantedException.WRITE_LOCKED);
    entry.CountInt++;
    }

    /**
    * Not to be called more than once by the locking transaction for a 
    * specific object.
    */
  public static synchronized void unlock(Object obj, long handle) {
    HandleEntry entry = (HandleEntry) TheHandleSet.get(handle);
    if (entry != null) {
      entry.CountInt--;
      if (entry.CountInt == 0)
        TheHandleSet.remove(entry);
      return;
      }
    entry = (HandleEntry) TheReferenceSet.referenceOf(obj);
    if (entry == null)
      return;
    entry.CountInt--;
    if (entry.CountInt == 0)
      TheReferenceSet.remove(entry);
    }

    /**
    * Only upgrades READ locks when there is only one transaction owning
    * the read lock.
    */
  public static synchronized void upgrade(Object obj, long handle) {
    HandleEntry entry = (HandleEntry) TheHandleSet.get(handle);
    if (entry == null)
      entry = (HandleEntry) TheReferenceSet.referenceOf(obj);
    if (entry.CountInt > 1)
      throw new LockNotGrantedException(
        obj,LockNotGrantedException.READ_LOCKED);
    entry.ModeInt = WRITE;
    }


  // Protected Methods
  // -----------------

  protected synchronized static void processQueue() {
    WeakHashReference ref = (WeakHashReference) TheReferenceQueue.poll();
    while (ref != null) {
      TheReferenceSet.remove(ref);
      ref = (WeakHashReference) TheReferenceQueue.poll();
      }
    }

  }//LockMgr

class HandleEntry extends WeakHashReference implements Handle {
  public long HandleLong;
  public int ModeInt;
  public int CountInt = 1;
  public HandleEntry(Object obj, long handle, int mode) {
    super(obj,LockMgr.TheReferenceQueue);
    HandleLong = handle;
    ModeInt = mode;
    }
  public long getHandle() {
    return HandleLong;
    }
  }


