// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

  /**
  * Indicates a lock could not be granted on an object.  The reasons
  * could be:<ul>
  * <li>No current transaction.
  * <li>A read lock is requested on an object which is write locked.
  * <li>A write lock is requested on an object which is already locked.
  * <li>The in-memory instance in question was write locked in a transaction
  * which was aborted.  It's state can not be quaranteed equals to it's last
  * committed state.
  * </ul>
  * @author Aaron Hansen
  * @see XL2Transaction#lock(Object,int)
  * @version 1.0.0
  */
public class LockNotGrantedException extends XL2Exception {

  // Public Fields
  // -------------
    /**
    * The in-memory instance in question was write locked in a transaction
    * which was aborted.  It's state can not be quaranteed equal to it's last
    * committed state.
    */
  public static String ABORTED = "Aborted Object";

    /**
    * A write lock was requested on an object which was already read locked.
    */
  public static String READ_LOCKED = "Read Locked";

    /**
    * A write lock was requested in a read-only transaction.
    */
  public static String READONLY_TRANSACTION = "Read Only Transaction";

    /**
    * A lock was requested on an object but the calling thread was not
    * joined to a XL2Transaction and there was no global XL2Transaction.
    */
  public static String TRANSACTION_REQUIRED = "Transaction Required.";

    /**
    * A lock was requested on an object which was already write locked.
    */
  public static String WRITE_LOCKED = "WriteLocked";


  // Protected Fields
  // ----------------

  protected String ReasonString = null;

  protected Object TheObject = null;


  // Constructors
  // ------------

  public LockNotGrantedException(Object object, String reason) {
    super(reason + ": " + String.valueOf(object));
    ReasonString = reason;
    TheObject = object;
    }


  // Public Methods
  // --------------

  public Object getObject() {
    return TheObject;
    }

    /**
    * See the constants above.
    */
  public String getReason() {
    return ReasonString;
    }


  }//LockNotGrantedException
