// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.lang.ref.*;
import xl2.odb.util.SoftHashReference;

  /**
  * Represents a first class object and the state regarding its storage.
  * @author Aaron Hansen
  * @version 1.0.0
  */
class Referent extends SoftHashReference implements Handle, XL2Constants {

  // Constructors
  // ------------

    /**
    * Default constructor.
    */
  public Referent(long handle,
                  long offset,
                  long refs,
                  int length,
                  Object object,
                  XL2Database db,
                  boolean stored) {
    super(object,ReferentMgr.refQueue());
    setXL2Database(db);
    setHandle(handle);
    setOffset(offset);
    setLength(length);
    setRefCount(refs);
    setStored(stored);
    }


  // XL2Database Property
  // -----------------

  protected XL2Database TheXL2Database = null;

  public XL2Database getXL2Database() {
    return TheXL2Database;
    }

  public void setXL2Database(XL2Database db) {
    TheXL2Database = db;
    }


  // Handle Property
  // ---------------

  protected long HandleLong = -1L;

  public long ensureHandle() throws java.io.IOException {
    if (HandleLong < 0)
      HandleLong = TheXL2Database.nextHandle();
    return HandleLong;
    }

  public long getHandle() {
    return HandleLong;
    }

  public void setHandle(long aLong) {
    HandleLong = aLong;
    }


  // Length Property
  // ---------------

  protected int LengthInt = -1;

  public int getLength() {
    return LengthInt;
    }

  public void setLength(int i) {
    LengthInt = i;
    }


  // Offset Property
  // ---------------

  protected long OffsetLong = -1L;

  public long getOffset() {
    return OffsetLong;
    }

  public void setOffset(long aLong) {
    OffsetLong = aLong;
    }


  // RefCount Property
  // -----------------

  protected long RefCountLong = 0L;

    /**
    * Returns new count.
    */
  public long decRefCount() {
    XL2TransactionMgr.currentTransaction().saveState(get());
    return --RefCountLong;
    }

  public long getRefCount() {
    return RefCountLong;
    }

    /**
    * Returns new count.
    */
  public long incRefCount() {
    XL2TransactionMgr.currentTransaction().saveState(get());
    return ++RefCountLong;
    }

  public void setRefCount(long l) {
    RefCountLong = l;
    }


  // Stored Property
  // ---------------

  protected boolean StoredBoolean = false;

  public boolean getStored() {
    return StoredBoolean;
    }

  public boolean isStored() {
    return StoredBoolean;
    }

  public void setStored(boolean b) {
    StoredBoolean = b;
    }


  // Public Methods
  // --------------

  public boolean equals(Object obj) {
    return (get() == obj);
    }

  public String toString() {
    return get().toString();
    }


  }//Referent
