// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import java.io.*;
import java.util.*;
import xl2.transaction.AbortFailedException;
import xl2.transaction.TransactionNotOpenException;

  /**
  * @author Aaron Hansen
  * @version 1.1.0
  */
class XL2Daemon extends Thread {

  // Protected Fields
  // ----------------

  protected boolean AliveBoolean = true;

  protected Compactor TheCompactor = null;

  protected XL2Database TheXL2Database = null;

  protected Object WaitObject = new Object();


  // Constructors
  // ------------

    /**
    * Default constructor.
    */
  public XL2Daemon(XL2Database db) {
    super();
    TheXL2Database = db;
    TheCompactor = new Compactor(db);
    this.setName("Daemon of " + TheXL2Database.PathString);
    this.setDaemon(true);
    this.setPriority(NORM_PRIORITY);
    }


  // Public Methods
  // --------------

    /**
    * Wake the daemon.
    */
  public void hello() {
    synchronized(WaitObject) {
      try {
        WaitObject.notify();
        } catch (Exception x) {}
      }
    }

    /**
    * Aborts expired transactions, garbage collects and compacts.
    */
  public void run() {
    XL2Transaction[] transactions = null;
    XL2Database db = TheXL2Database;
    XL2Transaction gcTrans = db.newTransaction();
    long nextExpiration = 0L;
    int loop = 0;
    long time = 0;
    boolean gcCompact = (db.getOpenMode() == db.OPEN_READ_WRITE) ;
    while (AliveBoolean) {
      transactions = db.transactions();
      Thread.yield();
      nextExpiration = 0;
      if ((AliveBoolean) && (transactions.length > 0)) {
        loop = transactions.length;
        while (AliveBoolean && (--loop >= 0)) {
          if (transactions[loop].isOpen()) {
            try { time = transactions[loop].checkExpiration(); }
            catch (AbortFailedException x) {db.log(null,x);}
            catch (TransactionNotOpenException x) {;}
            catch (Exception x) {db.log(null,x);}
            if ( (time > 0) 
              && ((nextExpiration == 0) || (time < nextExpiration)) )
              nextExpiration = time;
            }
          Thread.yield();
          }
        }
      if (AliveBoolean) {
        time = nextExpiration - System.currentTimeMillis();
        //The time will be < 0 if there are no transactions.
        //Yet, lets run every five seconds for no particular reason.
        if (time < - 5000)
          time = 5000;
        if ( (time > 1000) 
          && (gcCompact == true)
          && (db.CloseQueue.size() == 0)
          && (transactions.length == 0)) {
          try {
            if (!db.TheGarbageCollector.isEmpty()) {
              gcCompact = false;
              gcTrans.join();
              gcTrans.begin();
              db.TheGarbageCollector.gc();
              gcTrans.commit();
              gcCompact = true;
              Thread.yield();
              }
            else if (TheCompactor.hasWork()) {
              gcCompact = false;
              TheCompactor.begin();
              TheCompactor.commit();
              gcCompact = true;
              Thread.yield();
              }
            else synchronized (WaitObject) {
              try { WaitObject.wait(time); } catch (Exception x) {}
              }
            }
          catch (Exception x) {
            db.log(null,x);
            if (gcTrans.isOpen()) {
              try {
                gcTrans.abort();
                }
              catch (Exception ex) {
                db.log("Attempting to abort gc trans after exception.",ex);
                }
              }
            }
          }
        else if (time > 1000L) {
          synchronized (WaitObject) {
            try { WaitObject.wait(time); } catch (Exception x) {}
            }
          }
        }// if alive
      try {
        InvalidMgr.processQueue();
        Thread.yield();
        LockMgr.processQueue();
        Thread.yield();
        ReferentMgr.processQueue();
        Thread.yield();
        }
      catch (Exception x) {
        //Don't think this is possible...
        db.log(null,x);
        }
      }
    }

    /**
    * Stops the daemon, will not return until the daemon completes the run
    * method.
    */
  public void terminate() {
    synchronized(WaitObject) {
      try {
        AliveBoolean = false;
        WaitObject.notify();
        } catch (Exception x) {}
      }
    try { this.join(); } catch (Exception x) {}
    }


  }//XL2Daemon

/*
History
-------
v1.1.0, 2000-07-25
  - Removed waitMS and inlined it into run().
  - Now logs exceptions usine XL2Database.log.
  - Now any exception from gc or compact prevent any further collection
    or compaction.
  - GC and compaction no longer happen when there are active transactions.
*/
