// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb;

import xl2.transaction.TransactionMgr;
import xl2.transaction.TransactionRequiredException;

  /**
  * Static access to a thread's XL2Transaction.
  * @author Aaron Hansen
  * @version 1.0.0
  */
public class XL2TransactionMgr {

  // Public Fields
  // -------------

    /**
    * Lock type.
    */
  public static int READ = XL2Constants.READ;

    /**
    * Lock type.
    */
  public static int WRITE = XL2Constants.WRITE;


  // Public Methods
  // --------------

    /**
    * Calls the corresponding method on the XL2Transaction of the calling 
    * thread.
    */
  public static void abort() {
    TransactionMgr.abort();
    }

    /**
    * Calls the corresponding method on the XL2Transaction of the calling 
    * thread.
    */
  public static void begin() {
    TransactionMgr.begin();
    }

    /**
    * Calls the corresponding method on the XL2Transaction of the calling 
    * thread.
    */
  public static void commit() {
    TransactionMgr.commit();
    }

    /**
    * The XL2Transaction of the calling thread.
    * @see XL2Transaction#join()
    */
  public static XL2Transaction currentTransaction() {
    return (XL2Transaction) TransactionMgr.currentTransaction();
    }

    /**
    * The default XL2Transaction for any thread not explicitly joined to one.
    */
  public static XL2Transaction getGlobal() {
    return (XL2Transaction) TransactionMgr.getGlobal();
    }

    /**
    * Calls the corresponding method on the XL2Transaction of the calling 
    * thread.
    */
  public static void lock(Object obj, int mode) {
    XL2Transaction t = currentTransaction();
    if (t == null)
      throw new TransactionRequiredException();
    t.lock(obj, mode);
    }

    /**
    * Convenience function that calls lock(Object,int).
    */
  public static void readLock(Object obj) {
    lock(obj,READ);
    }

    /**
    * The default XL2Transaction for any thread not explicitly joined to one.
    */
  public static void setGlobal(XL2Transaction transaction) {
    TransactionMgr.setGlobal(transaction);
    }

    /**
    * Convenience function that calls lock(Object,int).
    */
  public static void writeLock(Object obj) {
    lock(obj,WRITE);
    }


  }//TransactionMgr
