// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb.io;

import java.io.*;

  /**
  * RandomAccessFile as an InputStream.
  * @author Aaron Hansen
  * @see RandomAccessFile
  * @version 1.0.0
  */
public class SeekableInputStream extends InputStream {

  // Protected Fields
  // ----------------

  protected RandomAccessFile TheFile = null;


  // Constructors
  // ------------

  public SeekableInputStream(File file, int bufferSize) 
    throws IOException {
    TheFile = new RandomAccessFile(file, "rw");
    }


  // Public Methods
  // --------------

    /**
    * Returns file length minus the current position.
    */
  public int available() throws IOException {
    long l = TheFile.length() - TheFile.getFilePointer();
    if (l > Integer.MAX_VALUE)
      return Integer.MAX_VALUE;
    else
      return (int) l;
    }

  public void close() throws IOException {
    TheFile.close();
    }

  public long length() throws IOException {
    return TheFile.length();
    }

  public long offset() throws IOException {
    return TheFile.getFilePointer();
    }

  public int read() throws IOException {
    return TheFile.read();
    }

  public int readInt() throws IOException {
	  int ch1 = read();
	  int ch2 = read();
	  int ch3 = read();
	  int ch4 = read();
	  if ((ch1 | ch2 | ch3 | ch4) < 0)
	     throw new EOFException();
	  return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
    }

  public long readLong() throws IOException {
	  return ((long)readInt() << 32) + (readInt() & 0xFFFFFFFFL);
    }

  public void seek(long offset) throws IOException {
    TheFile.seek(offset);
    }

  public long skip(long length) throws IOException {
    return TheFile.skipBytes((int)length);
    }

    /**
    * Calls toString of the random access file.
    */
  public String toString() {
    if (TheFile != null)
      return TheFile.toString();
    return super.toString();
    }


  // Protected Methods
  // -----------------

  protected void finalize() {
    try {
      close();
      } catch (Exception x) {}
    }


  }//SeekableInputStream
