package xl2.odb.io;

import java.io.*;

  /**
  * A RandomAccessFile as a buffered OutputStream.
  * @author Aaron Hansen
  * @see RandomAccessFile
  * @version 1.0.0
  */
public class SeekableOutputStream extends OutputStream {

  // Protected Fields
  // ----------------

  protected int LengthInt = 0;

  protected byte[] TheBuffer = null;

  protected RandomAccessFile TheFile = null;


  // Constructors
  // ------------

  public SeekableOutputStream(File file, int bufferSize) 
    throws FileNotFoundException {
    TheFile = new RandomAccessFile(file, "rw");
    TheBuffer = new byte[bufferSize];
    }


  // Public Methods
  // --------------

  public void close() throws IOException {
    flush();
    TheFile.close();
    }

  public void flush() throws IOException {
    if (LengthInt == 0)
      return;
    TheFile.write(TheBuffer,0,LengthInt);
    LengthInt = 0;
    }

  public FileDescriptor getFD() throws IOException {
    return TheFile.getFD();
    }

  public RandomAccessFile getFile() {
    return TheFile;
    }

  public long length() throws IOException {
    return TheFile.length();
    }

  public long offset() throws IOException {
    return TheFile.getFilePointer() + ((long)LengthInt);
    }

  public void seek(long offset) throws IOException {
    flush();
    TheFile.seek(offset);
    }

  public void setLength(long length) throws IOException {
    TheFile.setLength(length);
    }

    /**
    * Calls toString of the random access file.
    */
  public String toString() {
    if (TheFile != null)
      return TheFile.toString();
    return super.toString();
    }

  public void write(int i) throws IOException {
    if (LengthInt >= TheBuffer.length)
      flush();
    TheBuffer[LengthInt++] = (byte) (i & 0xff);
    }

  public void writeInt(int i) throws IOException {
    write((i >>> 24) & 0xFF);
    write((i >>> 16) & 0xFF);
    write((i >>>  8) & 0xFF);
    write((i >>>  0) & 0xFF);
    }

  public void writeLong(long l) throws IOException {
    write((int)(l >>> 56) & 0xFF);
    write((int)(l >>> 48) & 0xFF);
    write((int)(l >>> 40) & 0xFF);
    write((int)(l >>> 32) & 0xFF);
    write((int)(l >>> 24) & 0xFF);
    write((int)(l >>> 16) & 0xFF);
    write((int)(l >>>  8) & 0xFF);
    write((int)(l >>>  0) & 0xFF);
    }


  // Protected Methods
  // -----------------

  protected void finalize() {
    try {
      close();
      } catch (Exception x) {}
    }


  }//SeekableOutputStream
