// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb.util;

import xl2.collections.SkipSet;
import xl2.collections.node.SkipNode;

  /**
  * A hashCode ordered set that distinguishs duplicate hash codes 
  * from unequal instances.
  * @author Aaron Hansen
  * @version 1.0.0
  */
public class HashSet extends SkipSet {

  // Constructors
  // ------------

  public HashSet() {
    super();
    }


  // Public Methods
  // --------------

  public boolean add(Object obj) {
    if (obj == null)
      return false;
    SkipNode[] prev = predecessors(obj);
    if (prev[0] == null)
      append(prev, new SkipNode(levels(),obj));
    else if ((prev[0].Next[0] != null) 
      && (compare(prev[0].Next[0].Key,obj) == 0))
      return false;
    else if (prev[0].Key.hashCode() == obj.hashCode())
      append(prev, new SkipNode(prev[0].Next.length,obj));
    else
      append(prev, new SkipNode(levels(),obj));
    return true;
    }


  // Protected Methods
  // -----------------

  protected int compare(Object obj1, Object obj2) {
    int i = obj1.hashCode() - obj2.hashCode();
    if ((i == 0) && (obj1 != obj2))
      return -1;
    return i;
    }


  }//HashSet
