// Copyright (c) 2000 by Aaron Hansen. Use is subject to license terms.

package xl2.odb.util;

import java.lang.ref.Reference;
import xl2.collections.node.SkipNode;

  /**
  * Set of reference objects - where the hashCode of the reference equals that
  * of their referent.
  * @author Aaron Hansen
  * @version 1.0.0
  */
public class ReferenceSet extends HashSet {

  // Constructors
  // ------------

  public ReferenceSet() {
    super();
    }


  // Public Methods
  // --------------

  public boolean containsReference(Object obj) {
    if (obj == null)
      return false;
    return nodeOf(obj) != null;
    }

  public boolean containsReferent(Object obj) {
    if (obj == null)
      return false;
    return nodeOfReferent(obj) != null;
    }

  public Object referenceOf(Object obj) {
    SkipNode node = nodeOfReferent(obj);
    if (node != null)
      return node.Key;
    return null;
    }


  // Protected Methods
  // -----------------

  protected int compareReferent(Object obj1, Object obj2) {
    int i = obj1.hashCode() - System.identityHashCode(obj2);
    if ((i == 0) && (((Reference)obj1).get() != obj2))
      return -1;
    return i;
    }

  protected SkipNode nodeOfReferent(Object obj) {
    int level = Levels.length;
    int compared = -1;
    SkipNode current = null;
    while ((current == null) && (--level >= 0)) { //find starting level
      if ( (Levels[level] != null) 
        && ((compared = compareReferent(Levels[level].Key,obj)) < 0) ) {
        current = Levels[level];
        }
      }
    if (compared == 0) {
      if (level < 0) return Levels[0];
      else return Levels[level];
      }
    while (level >= 0) {  //now search
      while ((current.Next[level] != null) 
        &&  ((compared = compareReferent(current.Next[level].Key,obj)) < 0) ) {
        current = current.Next[level]; //go forward in level
        }
      if (compared == 0) return current.Next[level];
      level--;
      }
    return null;
    }


  }//ReferenceSet
