package xl2.util;

import java.util.*;
import java.io.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

  /**
  * Simple mailer for sending text messages.
  * @author aaron_hansen@yahoo.com
  */
public class Mailer {

  //protected fields
  //----------------

  protected PasswordAuthentication ThePasswordAuthentication = null;
  protected String PasswordString = null;
  protected Session TheSession = null;
  protected String UserString = null;


  //constructors
  //------------

  public Mailer(String smtpHost) {
    this(smtpHost,null,null);
    }

  public Mailer(String smtpHost, String user, String pass) {
    UserString = user;
    PasswordString = pass;
    Authenticator auth = null;
    if ((user == null) && (pass == null))
      auth = null;
    else if ((user == null) || (pass == null))
      throw new NullPointerException("Username or password was null.");
    else {
      ThePasswordAuthentication = new PasswordAuthentication(user,pass);
      auth = new Authenticator() {
        protected PasswordAuthentication getPasswordAuthentication() {
          return Mailer.this.ThePasswordAuthentication;
          }
        };
      }
    Properties props = new Properties();
    props.put("mail.smtp.host", smtpHost);
    TheSession = Session.getDefaultInstance(props, auth);
    }

  
  //public methods
  //--------------

  public static void main(String[] args) throws Exception {
    Mailer mailer = new Mailer("smtp.va.mediaone.net");
    mailer.send(new String[] {"ericfox@eworksfs.com"},
                null,
                null,
                "god@heaven.org",
                "asdf",
                "Stop picking your nose.");
    }

    /**
    * @param to Recipients.
    * @param cc Recipients.
    * @param bcc Recipients.
    * @param from Sender.
    * @param subject Message subject.
    * @param body Message body.
    * @param host SMTP host.
    */
  public synchronized void send(String[] to,
                                String[] cc,
                                String[] bcc,
                                String from,
                                String subject,
                                String body) {
    send(to,cc,bcc,from,subject,body,TheSession);
    }

    /**
    * @param to Recipients.
    * @param cc Recipients.
    * @param bcc Recipients.
    * @param from Sender.
    * @param subject Message subject.
    * @param body Message body.
    * @param session javax.mail.Session, can not be null.
    * @throws NullPointerException if the session is null.
    */
  public static void send(String[] to,
                          String[] cc,
                          String[] bcc,
                          String from,
                          String subject,
                          String body,
                          Session session) {
    if (session == null)
      throw new NullPointerException("Missing session.");
    if ((to == null) && (cc == null) && (bcc == null))
      throw new IllegalArgumentException("Missing recipients.");
    try {
      MimeMessage message = new MimeMessage(session);
      message.setSentDate(new Date());
      if (from != null)
        message.setFrom(new InternetAddress(from));
      InternetAddress[] addresses = 
        new InternetAddress[to.length];
      for (int i = to.length; --i >= 0; )
        addresses[i] = new InternetAddress(to[i]);
      message.setRecipients(Message.RecipientType.TO, addresses);
      if ((cc != null) && (cc.length > 0)) {
        addresses = new InternetAddress[cc.length];
        for (int i = cc.length; --i >= 0; )
          addresses[i] = new InternetAddress(cc[i]);
        message.setRecipients(Message.RecipientType.CC, addresses);
        }
      if ((bcc != null) && (bcc.length > 0)) {
        addresses = new InternetAddress[bcc.length];
        for (int i = bcc.length; --i >= 0; )
          addresses[i] = new InternetAddress(bcc[i]);
        message.setRecipients(Message.RecipientType.BCC, addresses);
        }
      if (subject != null)
        message.setSubject(subject);
      if (body != null)
        message.setText(body);
      Transport.send(message);
      } 
    catch (MessagingException mex) {
      mex.printStackTrace(System.err);
      System.err.println();
      Exception ex = mex;
      do {
        if (ex instanceof SendFailedException) {
          SendFailedException sfex = (SendFailedException)ex;
          Address[] invalid = sfex.getInvalidAddresses();
          if (invalid != null) {
            System.err.println("    ** Invalid Addresses");
            if (invalid != null) {
              for (int i = 0; i < invalid.length; i++) 
                System.err.println("   " + invalid[i]);
              }
            }
          Address[] validUnsent = sfex.getValidUnsentAddresses();
          if (validUnsent != null) {
            System.err.println("    ** Valid-Unsent Addresses");
            if (validUnsent != null) {
              for (int i = 0; i < validUnsent.length; i++) 
                System.err.println("   "+validUnsent[i]);
              }
            }
          }
          System.err.println();
        } 
      while ((ex = ((MessagingException)ex).getNextException()) != null);
      }
    catch (Exception x) {
      x.printStackTrace(System.err);
      }
    }


  }//class
