package xl2.util;

import java.io.*;
import java.util.*;

  /**
  * Misc String utilities.  No method will return null.
  */
public class Strings {

  //public methods
  //--------------

    /**
    * Extracts all digits from the specified string and returns
    * a new string of just the digits.  If the string is null, or
    * there are no digits, an empty string is returned.
    */
  public static String digits(String string) {
    if (string == null)
      return new String();
    StringBuffer buffer = new StringBuffer(16);
    int lengthInt = string.length();
    boolean hasDigitsBoolean = false;
    char c;
    for (int i = 0; i < lengthInt; i++) {
      c = string.charAt(i);
      if (Character.isDigit(c)) {
        buffer.append(c);
        hasDigitsBoolean = true;
        }
      }
    if (hasDigitsBoolean)
      return buffer.toString();
    else
      return new String();
    }

    /**
    * Extracts a field from the source string.  Will not return null.
    * @param field The field number, not index (1 majored).
    * @param src Source string.  Can not be null.
    * @param delim Field delimiter
    * @return The substring as defined by the parameters, will not return null.
    */
  public static String field(int field, String src, char delim) {
    int start = 0;
    for (int i = 1; i < field; i++) {
      start = src.indexOf(delim,start);
      start++;
      }
    int end = src.indexOf(delim,start);
    if (end >= start)
      return src.substring(start,end);
    else
      return src.substring(start);
    }

    /**
    * Using the specified delimiter, extracts all fields from the src
    * string and returns them in a vector.  Will not return null.
    */
  public static Vector fields(String src, char delim) {
    Vector strings = new Vector();
    if (src == null)
      return strings;
    char ch;
    int len = src.length();
    StringBuffer buf = new StringBuffer(len);
    for (int i = 0; i < len; i++) {
      ch = src.charAt(i);
      if (ch == delim) {
        strings.addElement(buf.toString());
        buf.setLength(0);
        }
      else
        buf.append(ch);
      }
    if (buf.length() > 0)
      strings.addElement(buf.toString());
    return strings;
    }

    /**
    * Trims leading and trailing whitespace, converts control characters
    * to whitespace, and reduces sequences of whitespace to a single space.
    * Will not return null.
    */
  public static String format(String field) {
    if (field == null)
      return new String();
    int len = field.length();
    StringBuffer out = new StringBuffer(len);
    int pos = 0;
    boolean ws = false; //whitespace
    char curr;
    while ((pos < len) && (field.charAt(pos) <= ' '))  //skip leading white
      pos++;
    while (pos < len) {
      curr = field.charAt(pos);
      if (curr > ' ') {
        if (ws) {
          out.append(' ');
          ws = false;
          }
        out.append(curr);
        }
      else 
        ws = true;
      pos++;
      }
    return out.toString();
    }

    /**
    * Ensures the string's length will not exceed the specified number
    * of characters.  Will not return null.
    * @param size Max size, or -1 for infinite size.
    * @return A string no longer than len if len was a positive number.
    */
  public static String max(int len, String s) {
    if ((s == null) || (len <= 0))
      return new String();
    if (s.length() > len)
      return s.substring(0,len);
    else
      return s;
    }

    /**
    * Guarantees the specified string is of at least the specified length.  If
    * the string is less than the specified length, it is appended or prepended 
    * with the specified character so that it's length equals the specified 
    * length.
    * @param src The original string to pad, if null a pad of said len 
                 is returned.
    * @param len The length of the padded string. If &lt;0 an empty string
    *            is returned.
    * @param fill The char to prepend to the string for padding.
    * @param prepend If true, the pad is inserted before source string.
    */
  public static String min(String src, 
                           int len, 
                           char fill,
                           boolean prepend) {
    if (len <= 0)
      return new String();
    if (src == null)
      return stringOf(fill,len);
    if (len < src.length())
      return src;
    else {
      StringBuffer buffer = new StringBuffer(len);
      int diffInt = len - src.length();
      if (prepend)
        buffer.append(stringOf(fill,diffInt));
      buffer.append(src);
      if (!prepend)
        buffer.append(stringOf(fill,diffInt));
      return buffer.toString();
      }
    }

    /**
    * Replaces all occurrances of the char with the sub string.
    * Will not return null.
    * @param src The string to search for replacements.
    * @param old The character to replace in the output string.
    * @param sub The string to substitute for the old char.
    */
  public static String replace(String src, char old, String sub) {
    if (src == null)
      return new String();
    char c;
    int i = 0;
    int len = src.length();
    StringBuffer buffer = new StringBuffer();
    while (i < len) {
      c = src.charAt(i);
      if (c == old)
        buffer.append(sub);
      else
        buffer.append(c);
      i++;
      }
    return buffer.toString();
    }

    /**
    * Replaces all occurrances of the char with the sub string.
    * Will not return null.
    * @param src The string to search for replacements.
    * @param old The string to replace in the output string.
    * @param sub The string to substitute for the old string.
    */
  public static String replace(String src, String old, String sub) {
    if (src == null)
      return new String();
    int oldlen = old.length();
    int srclen = src.length();
    int idx = 0;
    char c;
    StringBuffer buf = new StringBuffer(srclen);
    while (idx < srclen) {
      if (src.regionMatches(idx,old,0,oldlen)) {
        buf.append(sub);
        idx += oldlen;
        }
      else {
        buf.append(src.charAt(idx));
        idx++;
        }
      }
    return buf.toString();
    }

    /**
    * Creates a string of characters of the specified character and length.
    * Will not return null.
    */
  public static String stringOf(char c,int lengthInt) {
    if (lengthInt < 0)
      return new String();
    StringBuffer buffer = new StringBuffer(lengthInt);
    for (int i = lengthInt; --i >= 0;)
      buffer.append(c);
    return buffer.toString();
    }

    /**
    * Creates a string array from the specified vector by calling toString 
    * on its objects.  Will not return null.
    */
  public static String[] toStringArray(Vector vector) {
    if (vector == null) {
      String[] strings = {};
      return strings;
      }
    String[] strings = new String[vector.size()];
    int i = strings.length;
    while (--i >= 0)
      strings[i] = String.valueOf(vector.elementAt(i));
    return strings;
    }


  }//class
