package xl2.util;

import java.util.Vector;
import org.w3c.dom.*;

  /**
  * Misc XML routines.
  */
public class XmlTools {

  // Public Methods
  // --------------

    /**
    * The value of all direct child text nodes is returned.
    */
  public static String[] getAllText(Element element) {
    if (element == null)
      return null;
    element.normalize();
    NodeList list = element.getChildNodes();
    int len = list.getLength();
    Node node;
    Vector v = new Vector();
    for (int i = 0; i < len; i++) {
      node = list.item(i);
      if (node instanceof Text)
        v.addElement(node.getNodeValue());
      }
    String[] text = new String[v.size()];
    v.copyInto(text);
    return text;
    }

    /**
    * Returns the first direct child element with the specified name.
    */
  public static Element getChild(Node parent, String name) {
    if (parent == null)
      return null;
    NodeList list = parent.getChildNodes();
    int len = list.getLength();
    Node node;
    for (int i = 0; i < len; i++) {
      node = list.item(i);
      if ((node instanceof Element) && node.getNodeName().equals(name))
        return (Element) node;
      }
    return null;
    }

    /**
    * Returns All direct child elements with the specified name.
    */
  public static Element[] getChildren(Node parent, String name) {
    if (parent == null)
      return null;
    NodeList list = parent.getChildNodes();
    int len = list.getLength();
    Node node;
    Vector v = new Vector();;
    for (int i = 0; i < len; i++) {
      node = list.item(i);
      if ((node instanceof Element) && node.getNodeName().equals(name))
        v.addElement(node);
      }
    Element[] elements = new Element[v.size()];
    v.copyInto(elements);
    return elements;
    }

    /**
    * The value of the first direct child text node is returned.
    * @return null if non found, or the element is null.
    */
  public static String getText(Element element) {
    if (element == null)
      return null;
    element.normalize();
    NodeList list = element.getChildNodes();
    int len = list.getLength();
    Node node;
    for (int i = 0; i < len; i++) {
      node = list.item(i);
      if (node instanceof Text)
        return node.getNodeValue();
      }
    return null;
    }

    /**
    * The value of the first direct child text node of each element is returned.
    */
  public static String[] getText(NodeList elements) {
    if (elements == null)
      return null;
    int len = elements.getLength();
    Vector v = new Vector();
    String text;
    for (int i = 0; i < len; i++) {
      text = getText((Element)elements.item(i));
      if (text != null)
        v.addElement(text);
      }
    String[] all = new String[v.size()];
    v.copyInto(all);
    return all;
    }



  }//XmlTools
